"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const constants_1 = require("./constants");
const internal_1 = require("../internal");
const defaultConfig = {
    cacheExpiration: 500
};
const MainAdapter = internal_1.NowAdapter;
class NearDB {
    /**
     * Constructor to setup config, and path, and required checks.
     * @param config configuration to initialize NearDB instance
     */
    constructor(config) {
        /** Check if config exists */
        /** Overwrites config param with default configuration */
        this.config = Object.assign({}, defaultConfig, config);
        // TODO: define the type of storage in the config
        this.adapter = new MainAdapter(this.config);
        // Creates instanceid
        this.instanceId = internal_1.uuid();
    }
    /**
     * Static method to create a new instance of NearDB class.
     * @param config configuration to initialize NearDB instance
     * @returns an initialized instance of NearDB with the config
     */
    static database(config) {
        return new NearDB(Object.assign(defaultConfig, config));
    }
    /**
     * Returns a new instance of collection
     * @param key expects key for collection
     * @returns an instance of the collection class
     */
    collection(key) {
        return new Collection(this, key);
    }
}
// Constants used for document update
NearDB.field = {
    deleteValue: constants_1.default.deleteValue
};
exports.NearDB = NearDB;
class Collection {
    constructor(instance, key, path) {
        // Check if this is a reserved keyword
        if (internal_1.reservedKey(key)) {
            throw new Error(key + ': is a reserved keyword');
        }
        this.instance = instance;
        // Copy value of path before passing, to avoid polluting scope
        let newPath = path ? [...path] : [];
        newPath.push({
            type: 'collection',
            key
        });
        this.path = newPath;
    }
    doc(key) {
        return new Document(this.instance, key, this.path);
    }
    /**
     * Adds a document to a collection by generating an id for the doc
     * @param value expects payload to be stored for the document
     * @returns a promise for the payload of the saved doc
     */
    add(value) {
        return new Document(this.instance, internal_1.uuid(), this.path).set(value);
    }
}
exports.Collection = Collection;
class Document {
    /** String path for storage */
    constructor(instance, key, path) {
        // Check if this is a reserved keyword
        if (internal_1.reservedKey(key)) {
            throw new Error(key + ': is a reserved keyword');
        }
        // Copy value of path before passing, to avoid polluting scope
        let newPath = [...path];
        // Push new pathItem into the path array
        newPath.push({
            type: 'document',
            key
        });
        this.path = newPath;
        this.instance = instance;
        // Sets default cache value
        this.cache = new internal_1.Cache(this.instance.config.cacheExpiration);
    }
    /**
     * Creates a new instance of Collection
     * @param key expects key for collection
     * @returns an instance of the collection class
     */
    collection(key) {
        return new Collection(this.instance, key, this.path);
    }
    /**
     * Sets document data from the path provided in the scope.
     * @param value expects payload to be stored for the document
     * @returns payload of the document requested
     */
    set(value) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                internal_1.checkValidObject(value);
                let payload = yield this.instance.adapter.set(value, this.path);
                return payload;
            }
            catch (err) {
                throw err;
            }
        });
    }
    /**
     * Gets document data from the path provided in the scope.
     * @param options sets options on how to get documents
     * @returns payload of the document requested
     */
    get() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                let payload = yield this.instance.adapter.get(this.path);
                return payload;
            }
            catch (err) {
                throw err;
            }
        });
    }
    /**
     * Update some fields of the document without overwriting entire document
     * @param value expects payload to be stored for the document
     * @returns a promise for the payload requested
     */
    update(value) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                internal_1.checkValidObject(value);
                let payload = yield this.instance.adapter.update(value, this.path);
                return payload;
            }
            catch (err) {
                throw err;
            }
        });
    }
    /**
     * Removes document store in the path of instance
     * @returns payload of the document requested
     */
    delete() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                let payload = yield this.instance.adapter.remove(this.path);
                return payload;
            }
            catch (err) {
                throw err;
            }
        });
    }
}
exports.Document = Document;
//# sourceMappingURL=core.js.map